using CUDAapi

config_path = joinpath(@__DIR__, "ext.jl")
const previous_config_path = config_path * ".bak"

function write_ext(config)
    open(config_path, "w") do io
        println(io, "# autogenerated file, do not edit")
        for (key,val) in config
            println(io, "const $key = $(repr(val))")
        end
    end
end

function main()
    ispath(config_path) && mv(config_path, previous_config_path; force=true)
    config = Dict{Symbol,Any}(:configured => false)
    write_ext(config)


    ## discover stuff

    toolkit = CUDAapi.find_toolkit()

    for name in ("cublas", "cusolver", "cufft", "curand", "cudnn")
        lib = Symbol("lib$name")
        config[lib] = CUDAapi.find_cuda_library(name, toolkit)
        if config[lib] == nothing
            @warn "could not find $name, its functionality will be unavailable"
        end
    end


    ## (re)generate ext.jl

    function globals(mod)
        all_names = names(mod, all=true)
        filter(name-> !any(name .== [nameof(mod), Symbol("#eval"), :eval]), all_names)
    end

    if isfile(previous_config_path)
        @eval module Previous; include($previous_config_path); end
        previous_config = Dict{Symbol,Any}(name => getfield(Previous, name)
                                           for name in globals(Previous))

        if config == previous_config
            @info("CuArrays.jl has already been built for this toolchain, no need to rebuild")
            mv(previous_config_path, config_path; force=true)
            return
        end
    end

    config[:configured] = true
    write_ext(config)

    return
end

main()
